<?php
/*
 * Plugin Name: Advanced Order Export - product feed
 * Description: Product feed, CSV Format, requires Advanced Order Export For WooCommerce
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Woe_Product_Feed_CSV_Format {
	var $export_completed = false;
	var $include_all_products;
	var $allowed_products;

	function __construct() {
		//hooks for UI
		add_action("woe_settings_above_buttons", array($this, "draw_options") );

		//before start export
		add_filter('woe_settings_validate_defaults', function ($settings) {
			if( !empty($settings['custom_product_feed_format']) ) {
				$settings['format'] = "CSV";
				add_filter("woe_csv_custom_output_func", array($this, 'format'), 10, 7 );
				// we don't need orders at all!
				$settings['skip_empty_file'] = false;
				add_filter("woe_exported_file_is_empty", "__return_false");
				add_filter( 'woe_sql_get_order_ids_where', function($where, $settings ){
					$where = array( "1=0");
					return $where;
				},10,2);
			}
			return $settings;
		});
	}

	function draw_options($settings) {
		$selected = !empty($settings[ 'custom_product_feed_format' ]) ? 'checked': '';
		echo '<br><br>
		<input type=hidden name="settings[custom_product_feed_format]" value="0">
		<input type=checkbox name="settings[custom_product_feed_format]" value="1" '. $selected .'>
		Product Feed (CSV). Use button <b>Export [w/o progressbar]</b>.
		Warning, Preview doesn\'t work!<br>
		<br>';
	}

	public function trunc_chars($max_chars, $s) {
		return substr($s, 0, $max_chars);
	}

	public function format($custom_output, $handle, $data, $delimiter, $linebreak, $enclosure, $is_header) {
		if ($this->export_completed) {
			return true;
		}
		//to support checkbox "Export all products from the order"
		if( ! WC_Order_Export_Engine::$current_job_settings['all_products_from_order'] ) {
			if( !isset($this->allowed_products))
				$this->allowed_products = $this->get_product_ids( WC_Order_Export_Engine::$current_job_settings );
			$this->include_all_products   = false;
		}	
		else 	
			$this->include_all_products = true;

		//Headers
		$line = array("SKU","Name","Qty","EAN");
		fwrite($handle, implode($delimiter, $line) . $linebreak );// fputcsv?

		//see docs https://github.com/woocommerce/woocommerce/wiki/wc_get_products-and-WC_Product_Query#general
		$args = array(
		    'status' => array('publish'),
		    'limit' => -1,
			'return' => 'ids',
		);

		foreach(wc_get_products( $args ) as $pid) {
			if( !$this->include_all_products  AND !in_array($pid,$this->allowed_products) )
				continue;

			$main_product = wc_get_product($pid);
			if($main_product->get_type() == "variable") {
				$products = $main_product->get_available_variations('objects');
			} else {
				$products = array($main_product);
			}
			foreach($products as $product){
				$line = array();
				$line[] = $this->trunc_chars(25, $product->get_sku() );
				$line[] = $this->trunc_chars(60, $product->get_name() );
				$line[] = $this->trunc_chars(25, $product->get_stock_quantity() );
				$line[] = $this->trunc_chars(20, $product->get_meta("_wpm_gtin_code") );
				fwrite($handle, implode($delimiter, $line) . $linebreak );// fputcsv?
			}//each nested product
		}

		$this->export_completed = true;
		return true;
	}

	public function get_product_ids($settings) {
		global $wpdb;

		//custom taxonomies-- not implemented
		$taxonomy_where = "";

		$product_category_where = "";
		if ( $settings['product_categories'] ) {
			$cat_ids = array( 0 );
			foreach ( $settings['product_categories'] as $cat_id ) {
				$cat_ids[] = $cat_id;
				foreach ( get_term_children( $cat_id, 'product_cat' ) as $child_id ) {
					$cat_ids[] = $child_id;
				}
			}
			$cat_ids                = join( ',', $cat_ids );
			$product_category_where = "SELECT  DISTINCT object_id FROM {$wpdb->term_relationships} AS product_in_cat
						LEFT JOIN {$wpdb->term_taxonomy} AS product_category ON product_category.term_taxonomy_id = product_in_cat.term_taxonomy_id
						WHERE product_category.term_id IN ($cat_ids)
					";
			// get products and variations!
			$product_category_where = "AND ID IN (
					SELECT DISTINCT ID FROM {$wpdb->posts} AS product_category_variations WHERE post_parent<>0 AND post_parent IN ($product_category_where)
					UNION
					$product_category_where
				)";
		}


		$settings['products'] = WC_Order_Export_Data_Extractor::sql_get_filtered_product_list( $settings );

		// deep level still
		$exact_product_where = '';
		if ( $settings['products'] ) {
			$values = join( ",", $settings['products'] );
			$exact_product_where = "AND ID IN ($values)";
		}

		$exclude_product_where = '';
		if ( $settings['exclude_products'] ) {
			$values = join( ",", $settings['exclude_products'] );
			$exclude_product_where = "AND ID NOT IN ($values)";
		}

		$product_where = "SELECT DISTINCT ID FROM {$wpdb->posts} AS products WHERE products.post_type in ('product','product_variation') AND products.post_status<>'trash'";
		if($exact_product_where )
			$product_where .= $exact_product_where;
		if($product_category_where)
			$product_where .= $product_category_where;
		if($exclude_product_where )
			$product_where .= $exclude_product_where ;

		return $wpdb->get_col($product_where);
	}
}

new Woe_Product_Feed_CSV_Format();
